local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local abacus = AceLibrary("Abacus-2.0")
local L = AceLibrary("AceLocale-2.2"):new("NetWorthFu")

NetWorthFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
 
NetWorthFu.hasIcon = true;
NetWorthFu.clickableTooltip = true
NetWorthFu.frame = NetWorthFu:CreateBasicPluginFrame("NetWorthFuFrame")

local optionsTable = {
	handler = NetWorthFu,
	type = 'group',
	desc = L["DESCRIPTION"],
	args = {
		cashFormat = {
			order = 2,
			type = 'group',
			name = L["TEXT_CASHFORMAT"],
			desc = L["DESC_CASHFORMAT"],
			args = {
				barFormat = {
					order = 1,
					type = 'text',
        			usage = "",
					name = L["TEXT_FUDISPLAY"],
					desc = L["DESC_FUDISPLAY"],
					get = function() return NetWorthFu.db.profile.cashFormat.Bar end,
					set = function(v) NetWorthFu.db.profile.cashFormat.Bar = v; NetWorthFu:UpdateText() end,
					validate = {["Full"] = L["TEXT_FULLSTYLE"], ["Short"] = L["TEXT_SHORTSTYLE"], ["Condensed"] = L["TEXT_CONDSTYLE"], ["Graphical"] = L["TEXT_GRAPHICAL"]},
				},
				tooltipFormat = {
					order = 2,
					type = 'text',
        			usage = "",
					name = L["TEXT_TIPDISPLAY"],
					desc = L["DESC_TIPDISPLAY"],
					get = function() return NetWorthFu.db.profile.cashFormat.toolTip end,
					set = function(v) NetWorthFu.db.profile.cashFormat.toolTip = v; NetWorthFu:UpdateText() end,
					validate = {["Full"] = L["TEXT_FULLSTYLE"], ["Short"] = L["TEXT_SHORTSTYLE"], ["Condensed"] = L["TEXT_CONDSTYLE"]},
				},
			}
		},
		["-blank1-"] = {
			order = 4,
			type = 'header',
		},
		filters = {
			order = 10,
			type = 'group',
			name = L["TEXT_FILTER"],
			desc = L["DESC_FILTER"],
		},
	}
}

NetWorthFu:RegisterDB("NetWorthFuDB")
NetWorthFu:RegisterDefaults('profile', {
	cashFormat = { Bar = "Full", toolTip = "Full" },
	bankHiddenDefault = true,
});

function NetWorthFu:OnInitialize()
    NetWorthFu:FrameSetup()
end

function NetWorthFu:OnEnable()
    NetWorthFuFrameGoldIcon:ClearAllPoints()
    NetWorthFuFrameGoldIcon:SetPoint("RIGHT", NetWorthFuFrameSilverText, "LEFT", 0, -1)
    NetWorthFuFrameSilverIcon:ClearAllPoints()
    NetWorthFuFrameSilverIcon:SetPoint("RIGHT", NetWorthFuFrameCopperText, "LEFT", 0, -1)

    self:RegisterEvent("PLAYER_MONEY", "UpdateDisplay")
    self:RegisterEvent("PLAYER_TRADE_MONEY", "UpdateDisplay")
    self:RegisterEvent("TRADE_MONEY_CHANGED", "UpdateDisplay")
    self:RegisterEvent("SEND_MAIL_MONEY_CHANGED", "UpdateDisplay")
    self:RegisterEvent("SEND_MAIL_COD_CHANGED", "UpdateDisplay")
    self:RegisterEvent("GUILDBANKFRAME_OPENED","GuildBank")
    self:RegisterEvent("GUILDBANKFRAME_UPDATE_MONEY","GuildBank")
end

function NetWorthFu:GuildBank()
    local guildName = GetGuildInfo("player");
    if (NetWorthFu.db.realm[guildName] == nil) then
        NetWorthFu.db.realm[guildName] = {};
        NetWorthFu.db.realm[guildName].copper = GetGuildBankMoney();
        self.cached_options = nil;
        if (NetWorthFu.db.profile.bankHiddenDefault) then
            NetWorthFu.db.realm[guildName].show = false;
        else
            NetWorthFu.db.realm[guildName].show = true;
        end
    else
        NetWorthFu.db.realm[guildName].copper = GetGuildBankMoney();
    end
    NetWorthFu.db.realm[guildName].guild = true;
end

function NetWorthFu:OnTextUpdate()
    if (NetWorthFu.db.realm[UnitName("player")] == nil) then
        NetWorthFu.db.realm[UnitName("player")] = {};
        NetWorthFu.db.realm[UnitName("player")].copper = GetMoney();
        NetWorthFu.db.realm[UnitName("player")].show = true;
    else
        NetWorthFu.db.realm[UnitName("player")].copper = GetMoney();
    end
	if self.db.profile.cashFormat.Bar == "Graphical" then
		if self:IsIconShown() then
			self.db.profile.iconVisible = true
			self:HideIcon()
		end
		self.hasIcon = false
		NetWorthFuFrameIcon:Hide()
		NetWorthFuFrameText:Hide()
		local copper = self:GetNetWorth()
		local gold = floor(copper / 10000)
		local silver = mod(floor(copper / 100), 100)
		copper = mod(copper, 100)
		local width = 0
		if gold == 0 then
			NetWorthFuFrameGoldIcon:Hide()
			NetWorthFuFrameGoldText:Hide()
		else
			NetWorthFuFrameGoldIcon:Show()
			NetWorthFuFrameGoldText:Show()
			NetWorthFuFrameGoldText:SetWidth(0)
			NetWorthFuFrameGoldText:SetText(gold)
			width = width + NetWorthFuFrameGoldIcon:GetWidth() + NetWorthFuFrameGoldText:GetWidth()
		end
		if gold == 0 and silver == 0 then
			NetWorthFuFrameSilverIcon:Hide()
			NetWorthFuFrameSilverText:Hide()
		else
			NetWorthFuFrameSilverIcon:Show()
			NetWorthFuFrameSilverText:Show()
			NetWorthFuFrameSilverText:SetWidth(0)
			NetWorthFuFrameSilverText:SetText(silver)
			width = width + NetWorthFuFrameSilverIcon:GetWidth() + NetWorthFuFrameSilverText:GetWidth()
		end
		NetWorthFuFrameCopperIcon:Show()
		NetWorthFuFrameCopperText:Show()
		NetWorthFuFrameCopperText:SetWidth(0)
		NetWorthFuFrameCopperText:SetText(copper)
		width = width + NetWorthFuFrameCopperIcon:GetWidth() + NetWorthFuFrameCopperText:GetWidth()
		self.frame:SetWidth(width)
	else
		if not self.hasIcon then
			self.hasIcon = true
			if self.db.profile.iconVisible then
				self:ShowIcon()
			end
		end
		self.db.profile.iconVisible = false
		NetWorthFuFrameGoldIcon:Hide()
		NetWorthFuFrameSilverIcon:Hide()
		NetWorthFuFrameCopperIcon:Hide()
		NetWorthFuFrameGoldText:Hide()
		NetWorthFuFrameSilverText:Hide()
		NetWorthFuFrameCopperText:Hide()
		if NetWorthFu:IsTextShown() == true then
			NetWorthFuFrameText:Show()
		else
			NetWorthFuFrameText:Hide()
		end
		if self.db.profile.cashFormat.Bar == "Full" then
			self:SetText(abacus:FormatMoneyFull(self:GetNetWorth(), true))
		elseif self.db.profile.cashFormat.Bar == "Short" then
			self:SetText(abacus:FormatMoneyShort(self:GetNetWorth(), true))
		else
			self:SetText(abacus:FormatMoneyCondensed(self:GetNetWorth(), true))
		end
		self:CheckWidth(true)
	end
end

function NetWorthFu:OnTooltipUpdate()
	
	local cat = tablet:AddCategory(
					'text', L["TEXT_CHARACTERS"],
					'textR', 0,
					'textG', 1,
					'textB', 0,
					'columns', 2,
					'child_textR', 1, 'child_textG', 0.8, 'child_textB', 0
				)
				
	for name,data in pairs(NetWorthFu.db.realm) do
	    local name2 = name;
	    if (data.show) then
            if (data.guild == true) then
                name2 = "<" .. name .. ">";
            end
	        cat:AddLine(
		        'text', "    "..name2..":",
		        'text2', NetWorthFu:AbacusStyle(data.copper)
	        )
	    end
	end
end

function NetWorthFu:GetNetWorth()
    local netWorth = 0;
    for _,value in pairs(NetWorthFu.db.realm) do
        if (value.show) then
            netWorth = netWorth + value.copper;
        end
    end
    return netWorth;
end

function NetWorthFu:AbacusStyle(formattedCash, coloured)
	if self.db.profile.cashFormat.toolTip == "Full" then
		return abacus:FormatMoneyFull(formattedCash, true, coloured)
	elseif self.db.profile.cashFormat.toolTip == "Short" then
		return abacus:FormatMoneyShort(formattedCash, true, coloured)
	else
		return abacus:FormatMoneyCondensed(formattedCash, true, coloured)
	end
end

function NetWorthFu:OnMenuRequest()
	if (not self.cached_options) then
	    local filterTable = optionsTable.args.filters.args
	    filterTable = {}
	    optionsTable.args.filters.args = filterTable
	    
		self.cached_options = optionsTable
		
		for name,data in pairs(NetWorthFu.db.realm) do
		    local name = name;
		    local name2 = name;
		    if (data.guild == true) then
		        name2 = "<" .. name .. ">";
		    end
		    filterTable[name] = name;
		    
		    filterTable[name] = {}
			filterTable[name].type = 'toggle'
			filterTable[name].name = name2
			filterTable[name].desc = L["TEXT_FILTER"]..L["TEXT_FOR"]..name2..L["TEXT_FROMTOTALS"]
			filterTable[name].set  = function() NetWorthFu:ToggleFiltered(name) end
			filterTable[name].get  = function() return NetWorthFu:IsFiltered(name) end
		end
	end
	
	dewdrop:FeedAceOptionsTable(self.cached_options)
end

function NetWorthFu:SetFontSize(size)
	if NetWorthFuFrameGoldIcon == nil then
		self.fontSize = size
		return
	end
	NetWorthFuFrameGoldIcon:SetWidth(size)
	NetWorthFuFrameGoldIcon:SetHeight(size)
	NetWorthFuFrameSilverIcon:SetWidth(size)
	NetWorthFuFrameSilverIcon:SetHeight(size)
	NetWorthFuFrameCopperIcon:SetWidth(size)
	NetWorthFuFrameCopperIcon:SetHeight(size)
	self.iconFrame:SetWidth(16)
	self.iconFrame:SetHeight(16)
	local font,_,flags = NetWorthFuFrameGoldText:GetFont()
	if font ~= nil then
		NetWorthFuFrameGoldText:SetFont(font, size, flags)
		NetWorthFuFrameSilverText:SetFont(font, size, flags)
		NetWorthFuFrameCopperText:SetFont(font, size, flags)
		self.textFrame:SetFont(font, size)
	end
	self:UpdateText()
end

function NetWorthFu:FrameSetup()
	local frame = self.frame
	local icon = frame:CreateTexture("NetWorthFuFrameIcon", "ARTWORK")
	icon:SetWidth(16)
	icon:SetHeight(16)
	icon:SetPoint("LEFT", frame, "LEFT")
	
	icon:SetTexture("Interface\\MoneyFrame\\UI-MoneyIcons")
	icon:SetTexCoord(0, 0.25, 0, 1)
	
	self.iconFrame = icon

	local text = frame:CreateFontString("NetWorthFuFrameText", "OVERLAY")
	text:SetJustifyH("RIGHT")
	text:SetPoint("RIGHT", frame, "RIGHT", 0, 1)
	text:SetFontObject(GameFontNormal)
	self.textFrame = text

	self:SetIcon(true)

	local goldIcon = frame:CreateTexture("NetWorthFuFrameGoldIcon", "ARTWORK")
	goldIcon:SetWidth(16)
	goldIcon:SetHeight(16)
	goldIcon:SetTexture("Interface\\MoneyFrame\\UI-MoneyIcons")
	goldIcon:SetTexCoord(0, 0.25, 0, 1)

	local silverIcon = frame:CreateTexture("NetWorthFuFrameSilverIcon", "ARTWORK")
	silverIcon:SetWidth(16)
	silverIcon:SetHeight(16)
	silverIcon:SetTexture("Interface\\MoneyFrame\\UI-MoneyIcons")
	silverIcon:SetTexCoord(0.25, 0.5, 0, 1)

	local copperIcon = frame:CreateTexture("NetWorthFuFrameCopperIcon", "ARTWORK")
	copperIcon:SetWidth(16)
	copperIcon:SetHeight(16)
	copperIcon:SetTexture("Interface\\MoneyFrame\\UI-MoneyIcons")
	copperIcon:SetTexCoord(0.5, 0.75, 0, 1)

	local goldText = frame:CreateFontString("NetWorthFuFrameGoldText", "OVERLAY")
	goldText:SetJustifyH("RIGHT")
	goldText:SetPoint("RIGHT", goldIcon, "LEFT", 0, 1)
	goldText:SetFontObject(GameFontNormal)

	local silverText = frame:CreateFontString("NetWorthFuFrameSilverText", "OVERLAY")
	silverText:SetJustifyH("RIGHT")
	silverText:SetPoint("RIGHT", silverIcon, "LEFT", 0, 1)
	silverText:SetFontObject(GameFontNormal)

	local copperText = frame:CreateFontString("NetWorthFuFrameCopperText", "OVERLAY")
	copperText:SetJustifyH("RIGHT")
	copperText:SetPoint("RIGHT", copperIcon, "LEFT", 0, 1)
	copperText:SetFontObject(GameFontNormal)

	copperIcon:SetPoint("RIGHT", frame, "RIGHT")
	silverIcon:SetPoint("RIGHT", copperText, "LEFT")
	goldIcon:SetPoint("RIGHT", silverText, "LEFT")
end


function NetWorthFu:IsFiltered(name)
	return not NetWorthFu.db.realm[name].show
end

function NetWorthFu:ToggleFiltered(name)
	NetWorthFu.db.realm[name].show = not NetWorthFu.db.realm[name].show
	self:UpdateDisplay()
end